==> fred_prequel-0.2.3-py3-none-any.whl: prequel/tests/test_decoder.py <==
from datetime import datetime, timezone
from typing import List, Optional, Tuple
from unittest import TestCase

from fred_api.public_request.common_types_pb2 import (
    CannotBeProcessed as CannotBeProcessedMsg,
    LogEntryId as LogEntryIdMsg,
    OnStatusAction as OnStatusActionMsg,
    PublicRequest as PublicRequestMsg,
    PublicRequestDoesNotExist as PublicRequestDoesNotExistMsg,
    PublicRequestId as PublicRequestIdMsg,
    RegistrarId as RegistrarIdMsg,
    RegistryObjectReference as RegistryObjectReferenceMsg,
    RegistryObjectType as RegistryObjectTypeMsg,
    Status as StatusMsg,
)
from fred_api.public_request.service_public_request_info_grpc_pb2 import GetReply as GetReplyMsg
from fred_api.public_request.service_public_request_processing_grpc_pb2 import (
    ResolveReply as ResolveReplyMsg,
)
from google.protobuf import timestamp_pb2

from prequel.common import (
    PublicRequest,
    PublicRequestId,
    Reference,
    RegistryObjectType,
    RequestOnStatusAction,
    RequestStatus,
)
from prequel.decoder import PublicRequestDecoder
from prequel.exceptions import CannotBeProcessed, PublicRequestDoesNotExist

try:
    from pydantic.v1 import ValidationError
except ImportError:  # pragma: no cover
    from pydantic import ValidationError  # type: ignore[no-redef,assignment,unused-ignore]


class PublicRequestMsgFactory:
    """Helper class to create PublicRequest protocol buffer message."""

    REF_TYPE: int = RegistryObjectTypeMsg.Enum.domain
    REF_VALUE: str = "012-345-6789"
    REQ_TYPE: str = "AuthInfo"
    REGISTRAR: str = "REG-TEST"
    EMAIL: str = "info@example.org"
    CREATE_TIME: datetime = datetime(2023, 1, 1, 12, 0, 0, tzinfo=timezone.utc)
    RES_TIME: datetime = datetime(2023, 1, 15, 12, 0, 0, tzinfo=timezone.utc)
    STATUS: int = StatusMsg.Enum.resolved
    ACT_STATUS: int = OnStatusActionMsg.Enum.processed

    @classmethod
    def make(
        cls,
        id: int = 1,
        ref_type: int = REF_TYPE,
        ref_value: str = REF_VALUE,
        type: str = REQ_TYPE,
        registrar_id: str = REGISTRAR,
        email: str = EMAIL,
        created: Optional[datetime] = CREATE_TIME,
        resolved: Optional[datetime] = RES_TIME,
        status: int = STATUS,
        on_status_action: int = ACT_STATUS,
        log_ids: bool = True,  # create log entry item for event
    ) -> PublicRequestMsg:
        """Make gRPC message."""
        public_request_id = "Request ID " + str(id)
        timestamp = timestamp_pb2.Timestamp()
        creation_id = ""
        if created:
            timestamp.FromDatetime(created)
            if log_ids:
                creation_id = "Created"
        create_time = timestamp
        timestamp = timestamp_pb2.Timestamp()
        resolving_id = ""
        if resolved:
            timestamp.FromDatetime(resolved)
            if log_ids:
                resolving_id = "Resolved"
        resolve_time = timestamp
        return PublicRequestMsg(
            public_request_id=PublicRequestIdMsg(value=public_request_id),
            numeric_id=id,
            registry_object_reference=RegistryObjectReferenceMsg(type=ref_type, value=ref_value),
            type=type,
            registrar_id=RegistrarIdMsg(value=registrar_id),
            response_email=email,
            create_time=create_time,
            creation_id=LogEntryIdMsg(value=creation_id),
            resolve_time=resolve_time,
            resolving_id=LogEntryIdMsg(value=resolving_id),
            status=status,
            on_status_action=on_status_action,
        )

    @classmethod
    def default_src(cls) -> PublicRequest:
        """Model with default arguments for easy comparison in tests."""
        return PublicRequest(
            public_request_id=PublicRequestId("Request ID 1"),
            numeric_id=1,
            reference=Reference(type=RegistryObjectType("domain"), value=cls.REF_VALUE),
            type=cls.REQ_TYPE,
            registrar_id=cls.REGISTRAR,
            response_email=cls.EMAIL,
            create_time=cls.CREATE_TIME,
            creation_id="Created",
            resolve_time=cls.RES_TIME,
            resolving_id="Resolved",
            status=RequestStatus("resolved"),
            on_status_action=RequestOnStatusAction("processed"),
        )


class PublicRequestDecoderTest(TestCase):
    def setUp(self):
        self.decoder = PublicRequestDecoder()

    def test_public_request_ok(self):
        message = PublicRequestMsgFactory.make()
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertEqual(public_request, PublicRequestMsgFactory.default_src())

    def test_public_request_obj_unknown(self):
        message = PublicRequestMsgFactory.make(ref_type=0)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_obj_empty(self):
        message = PublicRequestMsgFactory.make(ref_value="")
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_registrar_empty(self):
        message = PublicRequestMsgFactory.make(registrar_id="")
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.registrar_id)

    def test_public_request_email_empty(self):
        message = PublicRequestMsgFactory.make(email="")
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.response_email)

    def test_public_request_create_empty(self):
        message = PublicRequestMsgFactory.make(created=None)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_resolve_empty(self):
        message = PublicRequestMsgFactory.make(resolved=None)
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.resolve_time)

    def test_public_request_log_id_none(self):
        message = PublicRequestMsgFactory.make(log_ids=False)
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.creation_id)
        self.assertIsNone(public_request.resolving_id)

    def test_public_request_status_unknown(self):
        message = PublicRequestMsgFactory.make(status=StatusMsg.Enum.undefined)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_status(self):
        data: List[Tuple[int, RequestStatus]] = [
            (StatusMsg.Enum.opened, RequestStatus.OPEN),
            (StatusMsg.Enum.resolved, RequestStatus.RESOLVED),
            (StatusMsg.Enum.invalidated, RequestStatus.INVALIDATED),
        ]
        for status, decoded in data:
            with self.subTest(status=status):
                message = PublicRequestMsgFactory.make(status=status)
                public_request: PublicRequest = self.decoder.decode(message)
                self.assertEqual(public_request.status, decoded)

    def test_public_request_create_actstatus_unknown(self):
        message = PublicRequestMsgFactory.make(on_status_action=OnStatusActionMsg.Enum.undefined)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_actstatus(self):
        data: List[Tuple[int, RequestOnStatusAction]] = [
            (OnStatusActionMsg.Enum.scheduled, RequestOnStatusAction.SCHEDULED),
            (OnStatusActionMsg.Enum.processed, RequestOnStatusAction.PROCESSED),
            (OnStatusActionMsg.Enum.failed, RequestOnStatusAction.FAILED),
        ]
        for status, decoded in data:
            with self.subTest(status=status):
                message = PublicRequestMsgFactory.make(on_status_action=status)
                public_request: PublicRequest = self.decoder.decode(message)
                self.assertEqual(public_request.on_status_action, decoded)

    def test_exception_public_request_does_not_exist(self):
        message = GetReplyMsg()
        message.exception.public_request_does_not_exist.CopyFrom(PublicRequestDoesNotExistMsg())
        self.assertIsInstance(
            self.decoder.decode_exception(message.exception), PublicRequestDoesNotExist
        )

    def test_exception_cannot_be_processed(self):
        message = ResolveReplyMsg()
        message.exception.cannot_be_processed.CopyFrom(CannotBeProcessedMsg())
        self.assertIsInstance(self.decoder.decode_exception(message.exception), CannotBeProcessed)

==> fred_prequel-1.0.0-py3-none-any.whl: prequel/tests/test_decoder.py <==
from datetime import datetime, timezone
from typing import List, Optional, Tuple
from unittest import TestCase

from fred_api.public_request.common_types_pb2 import (
    CannotBeProcessed as CannotBeProcessedMsg,
    LogEntryId as LogEntryIdMsg,
    OnStatusAction as OnStatusActionMsg,
    PublicRequest as PublicRequestMsg,
    PublicRequestDoesNotExist as PublicRequestDoesNotExistMsg,
    PublicRequestId as PublicRequestIdMsg,
    RegistrarId as RegistrarIdMsg,
    RegistryObjectReference as RegistryObjectReferenceMsg,
    RegistryObjectType as RegistryObjectTypeMsg,
    Status as StatusMsg,
)
from fred_api.public_request.service_public_request_info_grpc_pb2 import GetReply as GetReplyMsg
from fred_api.public_request.service_public_request_processing_grpc_pb2 import (
    ResolveReply as ResolveReplyMsg,
)
from google.protobuf import timestamp_pb2
from pydantic import ValidationError

from prequel.common import (
    PublicRequest,
    PublicRequestId,
    Reference,
    RegistryObjectType,
    RequestOnStatusAction,
    RequestStatus,
)
from prequel.decoder import PublicRequestDecoder
from prequel.exceptions import CannotBeProcessed, PublicRequestDoesNotExist


class PublicRequestMsgFactory:
    """Helper class to create PublicRequest protocol buffer message."""

    REF_TYPE: int = RegistryObjectTypeMsg.Enum.domain
    REF_VALUE: str = "012-345-6789"
    REQ_TYPE: str = "AuthInfo"
    REGISTRAR: str = "REG-TEST"
    EMAIL: str = "info@example.org"
    CREATE_TIME: datetime = datetime(2023, 1, 1, 12, 0, 0, tzinfo=timezone.utc)
    RES_TIME: datetime = datetime(2023, 1, 15, 12, 0, 0, tzinfo=timezone.utc)
    STATUS: int = StatusMsg.Enum.resolved
    ACT_STATUS: int = OnStatusActionMsg.Enum.processed

    @classmethod
    def make(
        cls,
        id: int = 1,
        ref_type: int = REF_TYPE,
        ref_value: str = REF_VALUE,
        type: str = REQ_TYPE,
        registrar_id: str = REGISTRAR,
        email: str = EMAIL,
        created: Optional[datetime] = CREATE_TIME,
        resolved: Optional[datetime] = RES_TIME,
        status: int = STATUS,
        on_status_action: int = ACT_STATUS,
        log_ids: bool = True,  # create log entry item for event
    ) -> PublicRequestMsg:
        """Make gRPC message."""
        public_request_id = "Request ID " + str(id)
        timestamp = timestamp_pb2.Timestamp()
        creation_id = ""
        if created:
            timestamp.FromDatetime(created)
            if log_ids:
                creation_id = "Created"
        create_time = timestamp
        timestamp = timestamp_pb2.Timestamp()
        resolving_id = ""
        if resolved:
            timestamp.FromDatetime(resolved)
            if log_ids:
                resolving_id = "Resolved"
        resolve_time = timestamp
        return PublicRequestMsg(
            public_request_id=PublicRequestIdMsg(value=public_request_id),
            numeric_id=id,
            registry_object_reference=RegistryObjectReferenceMsg(type=ref_type, value=ref_value),
            type=type,
            registrar_id=RegistrarIdMsg(value=registrar_id),
            response_email=email,
            create_time=create_time,
            creation_id=LogEntryIdMsg(value=creation_id),
            resolve_time=resolve_time,
            resolving_id=LogEntryIdMsg(value=resolving_id),
            status=status,
            on_status_action=on_status_action,
        )

    @classmethod
    def default_src(cls) -> PublicRequest:
        """Model with default arguments for easy comparison in tests."""
        return PublicRequest(
            public_request_id=PublicRequestId("Request ID 1"),
            numeric_id=1,
            reference=Reference(type=RegistryObjectType("domain"), value=cls.REF_VALUE),
            type=cls.REQ_TYPE,
            registrar_id=cls.REGISTRAR,
            response_email=cls.EMAIL,
            create_time=cls.CREATE_TIME,
            creation_id="Created",
            resolve_time=cls.RES_TIME,
            resolving_id="Resolved",
            status=RequestStatus("resolved"),
            on_status_action=RequestOnStatusAction("processed"),
        )


class PublicRequestDecoderTest(TestCase):
    def setUp(self):
        self.decoder = PublicRequestDecoder()

    def test_public_request_ok(self):
        message = PublicRequestMsgFactory.make()
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertEqual(public_request, PublicRequestMsgFactory.default_src())

    def test_public_request_obj_unknown(self):
        message = PublicRequestMsgFactory.make(ref_type=0)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_obj_empty(self):
        message = PublicRequestMsgFactory.make(ref_value="")
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_registrar_empty(self):
        message = PublicRequestMsgFactory.make(registrar_id="")
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.registrar_id)

    def test_public_request_email_empty(self):
        message = PublicRequestMsgFactory.make(email="")
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.response_email)

    def test_public_request_create_empty(self):
        message = PublicRequestMsgFactory.make(created=None)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_resolve_empty(self):
        message = PublicRequestMsgFactory.make(resolved=None)
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.resolve_time)

    def test_public_request_log_id_none(self):
        message = PublicRequestMsgFactory.make(log_ids=False)
        public_request: PublicRequest = self.decoder.decode(message)
        self.assertIsNone(public_request.creation_id)
        self.assertIsNone(public_request.resolving_id)

    def test_public_request_status_unknown(self):
        message = PublicRequestMsgFactory.make(status=StatusMsg.Enum.undefined)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_status(self):
        data: List[Tuple[int, RequestStatus]] = [
            (StatusMsg.Enum.opened, RequestStatus.OPEN),
            (StatusMsg.Enum.resolved, RequestStatus.RESOLVED),
            (StatusMsg.Enum.invalidated, RequestStatus.INVALIDATED),
        ]
        for status, decoded in data:
            with self.subTest(status=status):
                message = PublicRequestMsgFactory.make(status=status)
                public_request: PublicRequest = self.decoder.decode(message)
                self.assertEqual(public_request.status, decoded)

    def test_public_request_create_actstatus_unknown(self):
        message = PublicRequestMsgFactory.make(on_status_action=OnStatusActionMsg.Enum.undefined)
        with self.assertRaises(ValidationError):
            self.decoder.decode(message)

    def test_public_request_actstatus(self):
        data: List[Tuple[int, RequestOnStatusAction]] = [
            (OnStatusActionMsg.Enum.scheduled, RequestOnStatusAction.SCHEDULED),
            (OnStatusActionMsg.Enum.processed, RequestOnStatusAction.PROCESSED),
            (OnStatusActionMsg.Enum.failed, RequestOnStatusAction.FAILED),
        ]
        for status, decoded in data:
            with self.subTest(status=status):
                message = PublicRequestMsgFactory.make(on_status_action=status)
                public_request: PublicRequest = self.decoder.decode(message)
                self.assertEqual(public_request.on_status_action, decoded)

    def test_exception_public_request_does_not_exist(self):
        message = GetReplyMsg()
        message.exception.public_request_does_not_exist.CopyFrom(PublicRequestDoesNotExistMsg())
        self.assertIsInstance(
            self.decoder.decode_exception(message.exception), PublicRequestDoesNotExist
        )

    def test_exception_cannot_be_processed(self):
        message = ResolveReplyMsg()
        message.exception.cannot_be_processed.CopyFrom(CannotBeProcessedMsg())
        self.assertIsInstance(self.decoder.decode_exception(message.exception), CannotBeProcessed)
